// emoji.go
//
//      emoji.Println("@{:beer:}Example Text")
//
package emoji

import (
    "bytes"
    "errors"
    "fmt"
    "io"
    "log"
)

const (
    EscapeChar     = '@' // Escape character for emoji syntax
    ReplacePadding = " "
)

// Mapping from character to concrete escape code.
var CodeMap = map[string]string{
	":capricorn:": "\U00002651",
	":end:": "\U0001f51a",
	":no_mobile_phones:": "\U0001f4f5",
	":couple:": "\U0001f46b",
	":snowman:": "\U000026c4",
	":sunrise_over_mountains:": "\U0001f304",
	":suspension_railway:": "\U0001f69f",
	":arrows_counterclockwise:": "\U0001f504",
	":bug:": "\U0001f41b",
	":confused:": "\U0001f615",
	":dress:": "\U0001f457",
	":honeybee:": "\U0001f41d",
	":waning_crescent_moon:": "\U0001f318",
	":balloon:": "\U0001f388",
	":bus:": "\U0001f68c",
	":package:": "\U0001f4e6",
	":pencil2:": "\U0000270f",
	":rage:": "\U0001f621",
	":space_invader:": "\U0001f47e",
	":white_medium_small_square:": "\U000025fd",
	":fast_forward:": "\U000023e9",
	":rice_cracker:": "\U0001f358",
	":incoming_envelope:": "\U0001f4e8",
	":sa:": "\U0001f202",
	":womens:": "\U0001f6ba",
	":arrow_right:": "\U000027a1",
	":construction_worker:": "\U0001f477",
	":notes:": "\U0001f3b6",
	":goat:": "\U0001f410",
	":grey_question:": "\U00002754",
	":lantern:": "\U0001f3ee",
	":rice_scene:": "\U0001f391",
	":running:": "\U0001f3c3",
	":ferris_wheel:": "\U0001f3a1",
	":musical_score:": "\U0001f3bc",
	":sparkle:": "\U00002747",
	":wink:": "\U0001f609",
	":art:": "\U0001f3a8",
	":clock330:": "\U0001f55e",
	":minidisc:": "\U0001f4bd",
	":no_entry_sign:": "\U0001f6ab",
	":wind_chime:": "\U0001f390",
	":cyclone:": "\U0001f300",
	":herb:": "\U0001f33f",
	":leopard:": "\U0001f406",
	":banana:": "\U0001f34c",
	":handbag:": "\U0001f45c",
	":honey_pot:": "\U0001f36f",
	":ok:": "\U0001f197",
	":hearts:": "\U00002665",
	":passport_control:": "\U0001f6c2",
	":moyai:": "\U0001f5ff",
	":smile:": "\U0001f604",
	":tiger2:": "\U0001f405",
	":twisted_rightwards_arrows:": "\U0001f500",
	":children_crossing:": "\U0001f6b8",
	":cow:": "\U0001f42e",
	":point_up:": "\U0000261d",
	":house:": "\U0001f3e0",
	":man_with_turban:": "\U0001f473",
	":mountain_railway:": "\U0001f69e",
	":vibration_mode:": "\U0001f4f3",
	":blowfish:": "\U0001f421",
	":it:": "\U0001f1ee\U0001f1f9",
	":oden:": "\U0001f362",
	":clock3:": "\U0001f552",
	":lollipop:": "\U0001f36d",
	":train:": "\U0001f68b",
	":scissors:": "\U00002702",
	":triangular_ruler:": "\U0001f4d0",
	":wedding:": "\U0001f492",
	":flashlight:": "\U0001f526",
	":secret:": "\U00003299",
	":sushi:": "\U0001f363",
	":blue_car:": "\U0001f699",
	":cd:": "\U0001f4bf",
	":milky_way:": "\U0001f30c",
	":mortar_board:": "\U0001f393",
	":crown:": "\U0001f451",
	":speech_balloon:": "\U0001f4ac",
	":bento:": "\U0001f371",
	":grey_exclamation:": "\U00002755",
	":hotel:": "\U0001f3e8",
	":keycap_ten:": "\U0001f51f",
	":newspaper:": "\U0001f4f0",
	":outbox_tray:": "\U0001f4e4",
	":racehorse:": "\U0001f40e",
	":laughing:": "\U0001f606",
	":black_large_square:": "\U00002b1b",
	":books:": "\U0001f4da",
	":eight_spoked_asterisk:": "\U00002733",
	":heavy_check_mark:": "\U00002714",
	":m:": "\U000024c2",
	":wave:": "\U0001f44b",
	":bicyclist:": "\U0001f6b4",
	":cocktail:": "\U0001f378",
	":european_castle:": "\U0001f3f0",
	":point_down:": "\U0001f447",
	":tokyo_tower:": "\U0001f5fc",
	":battery:": "\U0001f50b",
	":dancer:": "\U0001f483",
	":repeat:": "\U0001f501",
	":ru:": "\U0001f1f7\U0001f1fa",
	":new_moon:": "\U0001f311",
	":church:": "\U000026ea",
	":date:": "\U0001f4c5",
	":earth_americas:": "\U0001f30e",
	":footprints:": "\U0001f463",
	":libra:": "\U0000264e",
	":mountain_cableway:": "\U0001f6a0",
	":small_red_triangle_down:": "\U0001f53b",
	":top:": "\U0001f51d",
	":sunglasses:": "\U0001f60e",
	":abcd:": "\U0001f521",
	":cl:": "\U0001f191",
	":ski:": "\U0001f3bf",
	":book:": "\U0001f4d6",
	":hourglass_flowing_sand:": "\U000023f3",
	":stuck_out_tongue_closed_eyes:": "\U0001f61d",
	":cold_sweat:": "\U0001f630",
	":headphones:": "\U0001f3a7",
	":confetti_ball:": "\U0001f38a",
	":gemini:": "\U0000264a",
	":new:": "\U0001f195",
	":pray:": "\U0001f64f",
	":watch:": "\U0000231a",
	":coffee:": "\U00002615",
	":ghost:": "\U0001f47b",
	":on:": "\U0001f51b",
	":pouch:": "\U0001f45d",
	":taxi:": "\U0001f695",
	":hocho:": "\U0001f52a",
	":yum:": "\U0001f60b",
	":heavy_plus_sign:": "\U00002795",
	":tada:": "\U0001f389",
	":arrow_heading_down:": "\U00002935",
	":clock530:": "\U0001f560",
	":poultry_leg:": "\U0001f357",
	":elephant:": "\U0001f418",
	":gb:": "\U0001f1ec\U0001f1e7",
	":mahjong:": "\U0001f004",
	":rice:": "\U0001f35a",
	":musical_note:": "\U0001f3b5",
	":beginner:": "\U0001f530",
	":small_red_triangle:": "\U0001f53a",
	":tomato:": "\U0001f345",
	":clock1130:": "\U0001f566",
	":japanese_castle:": "\U0001f3ef",
	":sun_with_face:": "\U0001f31e",
	":four:": "\U00000034\U000020e3",
	":microphone:": "\U0001f3a4",
	":tennis:": "\U0001f3be",
	":arrow_up_down:": "\U00002195",
	":cn:": "\U0001f1e8\U0001f1f3",
	":horse_racing:": "\U0001f3c7",
	":no_bicycles:": "\U0001f6b3",
	":snail:": "\U0001f40c",
	":free:": "\U0001f193",
	":beetle:": "\U0001f41e",
	":black_small_square:": "\U000025aa",
	":file_folder:": "\U0001f4c1",
	":hushed:": "\U0001f62f",
	":skull:": "\U0001f480",
	":ab:": "\U0001f18e",
	":rocket:": "\U0001f680",
	":sweet_potato:": "\U0001f360",
	":guitar:": "\U0001f3b8",
	":poodle:": "\U0001f429",
	":tulip:": "\U0001f337",
	":large_orange_diamond:": "\U0001f536",
	":-1:": "\U0001f44e",
	":chart_with_upwards_trend:": "\U0001f4c8",
	":de:": "\U0001f1e9\U0001f1ea",
	":grapes:": "\U0001f347",
	":ideograph_advantage:": "\U0001f250",
	":japanese_ogre:": "\U0001f479",
	":telephone:": "\U0000260e",
	":clock230:": "\U0001f55d",
	":hourglass:": "\U0000231b",
	":leftwards_arrow_with_hook:": "\U000021a9",
	":sparkler:": "\U0001f387",
	":black_joker:": "\U0001f0cf",
	":clock730:": "\U0001f562",
	":first_quarter_moon_with_face:": "\U0001f31b",
	":man:": "\U0001f468",
	":clock4:": "\U0001f553",
	":fishing_pole_and_fish:": "\U0001f3a3",
	":tophat:": "\U0001f3a9",
	":white_medium_square:": "\U000025fb",
	":mega:": "\U0001f4e3",
	":spaghetti:": "\U0001f35d",
	":dart:": "\U0001f3af",
	":girl:": "\U0001f467",
	":womans_hat:": "\U0001f452",
	":bullettrain_front:": "\U0001f685",
	":department_store:": "\U0001f3ec",
	":heartbeat:": "\U0001f493",
	":palm_tree:": "\U0001f334",
	":swimmer:": "\U0001f3ca",
	":yellow_heart:": "\U0001f49b",
	":arrow_upper_right:": "\U00002197",
	":clock2:": "\U0001f551",
	":high_heel:": "\U0001f460",
	":arrow_double_up:": "\U000023eb",
	":cry:": "\U0001f622",
	":dvd:": "\U0001f4c0",
	":e-mail:": "\U0001f4e7",
	":baby_bottle:": "\U0001f37c",
	":cool:": "\U0001f192",
	":floppy_disk:": "\U0001f4be",
	":iphone:": "\U0001f4f1",
	":minibus:": "\U0001f690",
	":rooster:": "\U0001f413",
	":three:": "\U00000033\U000020e3",
	":white_small_square:": "\U000025ab",
	":cancer:": "\U0000264b",
	":question:": "\U00002753",
	":sake:": "\U0001f376",
	":birthday:": "\U0001f382",
	":dog2:": "\U0001f415",
	":loudspeaker:": "\U0001f4e2",
	":arrow_up_small:": "\U0001f53c",
	":camel:": "\U0001f42b",
	":koala:": "\U0001f428",
	":mag_right:": "\U0001f50e",
	":soccer:": "\U000026bd",
	":bike:": "\U0001f6b2",
	":ear_of_rice:": "\U0001f33e",
	":shit:": "\U0001f4a9",
	":u7981:": "\U0001f232",
	":bath:": "\U0001f6c0",
	":baby:": "\U0001f476",
	":lock_with_ink_pen:": "\U0001f50f",
	":necktie:": "\U0001f454",
	":bikini:": "\U0001f459",
	":blush:": "\U0001f60a",
	":heartpulse:": "\U0001f497",
	":pig_nose:": "\U0001f43d",
	":straight_ruler:": "\U0001f4cf",
	":u6e80:": "\U0001f235",
	":gift:": "\U0001f381",
	":traffic_light:": "\U0001f6a5",
	":hibiscus:": "\U0001f33a",
	":couple_with_heart:": "\U0001f491",
	":pushpin:": "\U0001f4cc",
	":u6709:": "\U0001f236",
	":walking:": "\U0001f6b6",
	":grinning:": "\U0001f600",
	":hash:": "\U00000023\U000020e3",
	":radio_button:": "\U0001f518",
	":raised_hand:": "\U0000270b",
	":shaved_ice:": "\U0001f367",
	":barber:": "\U0001f488",
	":cat:": "\U0001f431",
	":heavy_exclamation_mark:": "\U00002757",
	":ice_cream:": "\U0001f368",
	":mask:": "\U0001f637",
	":pig2:": "\U0001f416",
	":triangular_flag_on_post:": "\U0001f6a9",
	":arrow_upper_left:": "\U00002196",
	":bee:": "\U0001f41d",
	":beer:": "\U0001f37a",
	":black_nib:": "\U00002712",
	":exclamation:": "\U00002757",
	":dog:": "\U0001f436",
	":fire:": "\U0001f525",
	":ant:": "\U0001f41c",
	":broken_heart:": "\U0001f494",
	":chart:": "\U0001f4b9",
	":clock1:": "\U0001f550",
	":bomb:": "\U0001f4a3",
	":virgo:": "\U0000264d",
	":a:": "\U0001f170",
	":fork_and_knife:": "\U0001f374",
	":copyright:": "\U000000a9",
	":curly_loop:": "\U000027b0",
	":full_moon:": "\U0001f315",
	":shoe:": "\U0001f45e",
	":european_post_office:": "\U0001f3e4",
	":ng:": "\U0001f196",
	":office:": "\U0001f3e2",
	":raising_hand:": "\U0001f64b",
	":revolving_hearts:": "\U0001f49e",
	":aquarius:": "\U00002652",
	":electric_plug:": "\U0001f50c",
	":meat_on_bone:": "\U0001f356",
	":mens:": "\U0001f6b9",
	":briefcase:": "\U0001f4bc",
	":ship:": "\U0001f6a2",
	":anchor:": "\U00002693",
	":ballot_box_with_check:": "\U00002611",
	":bear:": "\U0001f43b",
	":beers:": "\U0001f37b",
	":dromedary_camel:": "\U0001f42a",
	":nut_and_bolt:": "\U0001f529",
	":construction:": "\U0001f6a7",
	":golf:": "\U000026f3",
	":toilet:": "\U0001f6bd",
	":blue_book:": "\U0001f4d8",
	":boom:": "\U0001f4a5",
	":deciduous_tree:": "\U0001f333",
	":kissing_closed_eyes:": "\U0001f61a",
	":smiley_cat:": "\U0001f63a",
	":fuelpump:": "\U000026fd",
	":kiss:": "\U0001f48b",
	":clock10:": "\U0001f559",
	":sheep:": "\U0001f411",
	":white_flower:": "\U0001f4ae",
	":boar:": "\U0001f417",
	":currency_exchange:": "\U0001f4b1",
	":facepunch:": "\U0001f44a",
	":flower_playing_cards:": "\U0001f3b4",
	":person_frowning:": "\U0001f64d",
	":poop:": "\U0001f4a9",
	":satisfied:": "\U0001f606",
	":8ball:": "\U0001f3b1",
	":disappointed_relieved:": "\U0001f625",
	":panda_face:": "\U0001f43c",
	":ticket:": "\U0001f3ab",
	":us:": "\U0001f1fa\U0001f1f8",
	":waxing_crescent_moon:": "\U0001f312",
	":dragon:": "\U0001f409",
	":gun:": "\U0001f52b",
	":mount_fuji:": "\U0001f5fb",
	":new_moon_with_face:": "\U0001f31a",
	":star2:": "\U0001f31f",
	":grimacing:": "\U0001f62c",
	":confounded:": "\U0001f616",
	":congratulations:": "\U00003297",
	":custard:": "\U0001f36e",
	":frowning:": "\U0001f626",
	":maple_leaf:": "\U0001f341",
	":police_car:": "\U0001f693",
	":cloud:": "\U00002601",
	":jeans:": "\U0001f456",
	":fish:": "\U0001f41f",
	":wavy_dash:": "\U00003030",
	":clock5:": "\U0001f554",
	":santa:": "\U0001f385",
	":japan:": "\U0001f5fe",
	":oncoming_taxi:": "\U0001f696",
	":whale:": "\U0001f433",
	":arrow_forward:": "\U000025b6",
	":kissing_heart:": "\U0001f618",
	":bullettrain_side:": "\U0001f684",
	":fearful:": "\U0001f628",
	":moneybag:": "\U0001f4b0",
	":runner:": "\U0001f3c3",
	":mailbox:": "\U0001f4eb",
	":sandal:": "\U0001f461",
	":zzz:": "\U0001f4a4",
	":apple:": "\U0001f34e",
	":arrow_heading_up:": "\U00002934",
	":family:": "\U0001f46a",
	":heavy_minus_sign:": "\U00002796",
	":saxophone:": "\U0001f3b7",
	":u5272:": "\U0001f239",
	":black_square_button:": "\U0001f532",
	":bouquet:": "\U0001f490",
	":love_letter:": "\U0001f48c",
	":metro:": "\U0001f687",
	":small_blue_diamond:": "\U0001f539",
	":thought_balloon:": "\U0001f4ad",
	":arrow_up:": "\U00002b06",
	":no_pedestrians:": "\U0001f6b7",
	":smirk:": "\U0001f60f",
	":blue_heart:": "\U0001f499",
	":large_blue_diamond:": "\U0001f537",
	":vs:": "\U0001f19a",
	":v:": "\U0000270c",
	":wheelchair:": "\U0000267f",
	":couplekiss:": "\U0001f48f",
	":tent:": "\U000026fa",
	":purple_heart:": "\U0001f49c",
	":relaxed:": "\U0000263a",
	":accept:": "\U0001f251",
	":green_heart:": "\U0001f49a",
	":pouting_cat:": "\U0001f63e",
	":tram:": "\U0001f68a",
	":bangbang:": "\U0000203c",
	":collision:": "\U0001f4a5",
	":convenience_store:": "\U0001f3ea",
	":person_with_blond_hair:": "\U0001f471",
	":uk:": "\U0001f1ec\U0001f1e7",
	":peach:": "\U0001f351",
	":tired_face:": "\U0001f62b",
	":bread:": "\U0001f35e",
	":mailbox_closed:": "\U0001f4ea",
	":open_mouth:": "\U0001f62e",
	":pig:": "\U0001f437",
	":put_litter_in_its_place:": "\U0001f6ae",
	":u7a7a:": "\U0001f233",
	":bulb:": "\U0001f4a1",
	":clock9:": "\U0001f558",
	":envelope_with_arrow:": "\U0001f4e9",
	":pisces:": "\U00002653",
	":baggage_claim:": "\U0001f6c4",
	":egg:": "\U0001f373",
	":sweat_smile:": "\U0001f605",
	":boat:": "\U000026f5",
	":fr:": "\U0001f1eb\U0001f1f7",
	":heavy_division_sign:": "\U00002797",
	":muscle:": "\U0001f4aa",
	":paw_prints:": "\U0001f43e",
	":arrow_left:": "\U00002b05",
	":black_circle:": "\U000026ab",
	":kissing_smiling_eyes:": "\U0001f619",
	":star:": "\U00002b50",
	":steam_locomotive:": "\U0001f682",
	":1234:": "\U0001f522",
	":clock130:": "\U0001f55c",
	":kr:": "\U0001f1f0\U0001f1f7",
	":monorail:": "\U0001f69d",
	":school:": "\U0001f3eb",
	":seven:": "\U00000037\U000020e3",
	":baby_chick:": "\U0001f424",
	":bridge_at_night:": "\U0001f309",
	":hotsprings:": "\U00002668",
	":rose:": "\U0001f339",
	":love_hotel:": "\U0001f3e9",
	":princess:": "\U0001f478",
	":ramen:": "\U0001f35c",
	":scroll:": "\U0001f4dc",
	":tropical_fish:": "\U0001f420",
	":heart_eyes_cat:": "\U0001f63b",
	":information_desk_person:": "\U0001f481",
	":mouse:": "\U0001f42d",
	":no_smoking:": "\U0001f6ad",
	":post_office:": "\U0001f3e3",
	":stars:": "\U0001f320",
	":arrow_double_down:": "\U000023ec",
	":unlock:": "\U0001f513",
	":arrow_backward:": "\U000025c0",
	":hand:": "\U0000270b",
	":hospital:": "\U0001f3e5",
	":ocean:": "\U0001f30a",
	":mountain_bicyclist:": "\U0001f6b5",
	":octopus:": "\U0001f419",
	":sos:": "\U0001f198",
	":dizzy_face:": "\U0001f635",
	":tongue:": "\U0001f445",
	":train2:": "\U0001f686",
	":checkered_flag:": "\U0001f3c1",
	":orange_book:": "\U0001f4d9",
	":sound:": "\U0001f509",
	":aerial_tramway:": "\U0001f6a1",
	":bell:": "\U0001f514",
	":dragon_face:": "\U0001f432",
	":flipper:": "\U0001f42c",
	":ok_woman:": "\U0001f646",
	":performing_arts:": "\U0001f3ad",
	":postal_horn:": "\U0001f4ef",
	":clock1030:": "\U0001f565",
	":email:": "\U00002709",
	":green_book:": "\U0001f4d7",
	":point_up_2:": "\U0001f446",
	":high_brightness:": "\U0001f506",
	":running_shirt_with_sash:": "\U0001f3bd",
	":bookmark:": "\U0001f516",
	":sob:": "\U0001f62d",
	":arrow_lower_right:": "\U00002198",
	":point_left:": "\U0001f448",
	":purse:": "\U0001f45b",
	":sparkles:": "\U00002728",
	":black_medium_small_square:": "\U000025fe",
	":pound:": "\U0001f4b7",
	":rabbit:": "\U0001f430",
	":woman:": "\U0001f469",
	":negative_squared_cross_mark:": "\U0000274e",
	":open_book:": "\U0001f4d6",
	":smiling_imp:": "\U0001f608",
	":spades:": "\U00002660",
	":baseball:": "\U000026be",
	":fountain:": "\U000026f2",
	":joy:": "\U0001f602",
	":lipstick:": "\U0001f484",
	":partly_sunny:": "\U000026c5",
	":ram:": "\U0001f40f",
	":red_circle:": "\U0001f534",
	":cop:": "\U0001f46e",
	":green_apple:": "\U0001f34f",
	":registered:": "\U000000ae",
	":+1:": "\U0001f44d",
	":crying_cat_face:": "\U0001f63f",
	":innocent:": "\U0001f607",
	":mobile_phone_off:": "\U0001f4f4",
	":underage:": "\U0001f51e",
	":dolphin:": "\U0001f42c",
	":busts_in_silhouette:": "\U0001f465",
	":umbrella:": "\U00002614",
	":angel:": "\U0001f47c",
	":small_orange_diamond:": "\U0001f538",
	":sunflower:": "\U0001f33b",
	":link:": "\U0001f517",
	":notebook:": "\U0001f4d3",
	":oncoming_bus:": "\U0001f68d",
	":bookmark_tabs:": "\U0001f4d1",
	":calendar:": "\U0001f4c6",
	":izakaya_lantern:": "\U0001f3ee",
	":mans_shoe:": "\U0001f45e",
	":name_badge:": "\U0001f4db",
	":closed_lock_with_key:": "\U0001f510",
	":fist:": "\U0000270a",
	":id:": "\U0001f194",
	":ambulance:": "\U0001f691",
	":musical_keyboard:": "\U0001f3b9",
	":ribbon:": "\U0001f380",
	":seedling:": "\U0001f331",
	":tv:": "\U0001f4fa",
	":football:": "\U0001f3c8",
	":nail_care:": "\U0001f485",
	":seat:": "\U0001f4ba",
	":alarm_clock:": "\U000023f0",
	":money_with_wings:": "\U0001f4b8",
	":relieved:": "\U0001f60c",
	":womans_clothes:": "\U0001f45a",
	":lips:": "\U0001f444",
	":clubs:": "\U00002663",
	":house_with_garden:": "\U0001f3e1",
	":sunrise:": "\U0001f305",
	":monkey:": "\U0001f412",
	":six:": "\U00000036\U000020e3",
	":smiley:": "\U0001f603",
	":feet:": "\U0001f43e",
	":waning_gibbous_moon:": "\U0001f316",
	":yen:": "\U0001f4b4",
	":baby_symbol:": "\U0001f6bc",
	":signal_strength:": "\U0001f4f6",
	":boy:": "\U0001f466",
	":busstop:": "\U0001f68f",
	":computer:": "\U0001f4bb",
	":night_with_stars:": "\U0001f303",
	":older_woman:": "\U0001f475",
	":parking:": "\U0001f17f",
	":trumpet:": "\U0001f3ba",
	":100:": "\U0001f4af",
	":sweat_drops:": "\U0001f4a6",
	":wc:": "\U0001f6be",
	":b:": "\U0001f171",
	":cupid:": "\U0001f498",
	":five:": "\U00000035\U000020e3",
	":part_alternation_mark:": "\U0000303d",
	":snowboarder:": "\U0001f3c2",
	":warning:": "\U000026a0",
	":white_large_square:": "\U00002b1c",
	":zap:": "\U000026a1",
	":arrow_down_small:": "\U0001f53d",
	":clock430:": "\U0001f55f",
	":expressionless:": "\U0001f611",
	":phone:": "\U0000260e",
	":roller_coaster:": "\U0001f3a2",
	":lemon:": "\U0001f34b",
	":one:": "\U00000031\U000020e3",
	":christmas_tree:": "\U0001f384",
	":hankey:": "\U0001f4a9",
	":hatched_chick:": "\U0001f425",
	":u7533:": "\U0001f238",
	":large_blue_circle:": "\U0001f535",
	":up:": "\U0001f199",
	":wine_glass:": "\U0001f377",
	":x:": "\U0000274c",
	":nose:": "\U0001f443",
	":rewind:": "\U000023ea",
	":two_hearts:": "\U0001f495",
	":envelope:": "\U00002709",
	":oncoming_automobile:": "\U0001f698",
	":ophiuchus:": "\U000026ce",
	":ring:": "\U0001f48d",
	":tropical_drink:": "\U0001f379",
	":turtle:": "\U0001f422",
	":crescent_moon:": "\U0001f319",
	":koko:": "\U0001f201",
	":microscope:": "\U0001f52c",
	":rugby_football:": "\U0001f3c9",
	":smoking:": "\U0001f6ac",
	":anger:": "\U0001f4a2",
	":aries:": "\U00002648",
	":city_sunset:": "\U0001f306",
	":clock1230:": "\U0001f567",
	":mailbox_with_no_mail:": "\U0001f4ed",
	":movie_camera:": "\U0001f3a5",
	":pager:": "\U0001f4df",
	":zero:": "\U00000030\U000020e3",
	":bank:": "\U0001f3e6",
	":eight_pointed_black_star:": "\U00002734",
	":knife:": "\U0001f52a",
	":u7121:": "\U0001f21a",
	":customs:": "\U0001f6c3",
	":melon:": "\U0001f348",
	":rowboat:": "\U0001f6a3",
	":corn:": "\U0001f33d",
	":eggplant:": "\U0001f346",
	":heart_decoration:": "\U0001f49f",
	":rotating_light:": "\U0001f6a8",
	":round_pushpin:": "\U0001f4cd",
	":cat2:": "\U0001f408",
	":chocolate_bar:": "\U0001f36b",
	":no_bell:": "\U0001f515",
	":radio:": "\U0001f4fb",
	":droplet:": "\U0001f4a7",
	":hamburger:": "\U0001f354",
	":fire_engine:": "\U0001f692",
	":heart:": "\U00002764",
	":potable_water:": "\U0001f6b0",
	":telephone_receiver:": "\U0001f4de",
	":dash:": "\U0001f4a8",
	":globe_with_meridians:": "\U0001f310",
	":guardsman:": "\U0001f482",
	":heavy_multiplication_x:": "\U00002716",
	":chart_with_downwards_trend:": "\U0001f4c9",
	":imp:": "\U0001f47f",
	":earth_asia:": "\U0001f30f",
	":mouse2:": "\U0001f401",
	":notebook_with_decorative_cover:": "\U0001f4d4",
	":telescope:": "\U0001f52d",
	":trolleybus:": "\U0001f68e",
	":card_index:": "\U0001f4c7",
	":euro:": "\U0001f4b6",
	":dollar:": "\U0001f4b5",
	":fax:": "\U0001f4e0",
	":mailbox_with_mail:": "\U0001f4ec",
	":raised_hands:": "\U0001f64c",
	":disappointed:": "\U0001f61e",
	":foggy:": "\U0001f301",
	":person_with_pouting_face:": "\U0001f64e",
	":statue_of_liberty:": "\U0001f5fd",
	":dolls:": "\U0001f38e",
	":light_rail:": "\U0001f688",
	":pencil:": "\U0001f4dd",
	":speak_no_evil:": "\U0001f64a",
	":calling:": "\U0001f4f2",
	":clock830:": "\U0001f563",
	":cow2:": "\U0001f404",
	":hear_no_evil:": "\U0001f649",
	":scream_cat:": "\U0001f640",
	":smile_cat:": "\U0001f638",
	":tractor:": "\U0001f69c",
	":clock11:": "\U0001f55a",
	":doughnut:": "\U0001f369",
	":hammer:": "\U0001f528",
	":loop:": "\U000027bf",
	":moon:": "\U0001f314",
	":soon:": "\U0001f51c",
	":cinema:": "\U0001f3a6",
	":factory:": "\U0001f3ed",
	":flushed:": "\U0001f633",
	":mute:": "\U0001f507",
	":neutral_face:": "\U0001f610",
	":scorpius:": "\U0000264f",
	":wolf:": "\U0001f43a",
	":clapper:": "\U0001f3ac",
	":joy_cat:": "\U0001f639",
	":pensive:": "\U0001f614",
	":sleeping:": "\U0001f634",
	":credit_card:": "\U0001f4b3",
	":leo:": "\U0000264c",
	":man_with_gua_pi_mao:": "\U0001f472",
	":open_hands:": "\U0001f450",
	":tea:": "\U0001f375",
	":arrow_down:": "\U00002b07",
	":nine:": "\U00000039\U000020e3",
	":punch:": "\U0001f44a",
	":slot_machine:": "\U0001f3b0",
	":clap:": "\U0001f44f",
	":information_source:": "\U00002139",
	":tiger:": "\U0001f42f",
	":city_sunrise:": "\U0001f307",
	":dango:": "\U0001f361",
	":thumbsdown:": "\U0001f44e",
	":u6307:": "\U0001f22f",
	":curry:": "\U0001f35b",
	":cherries:": "\U0001f352",
	":clock6:": "\U0001f555",
	":clock7:": "\U0001f556",
	":older_man:": "\U0001f474",
	":oncoming_police_car:": "\U0001f694",
	":syringe:": "\U0001f489",
	":heavy_dollar_sign:": "\U0001f4b2",
	":open_file_folder:": "\U0001f4c2",
	":arrow_right_hook:": "\U000021aa",
	":articulated_lorry:": "\U0001f69b",
	":dancers:": "\U0001f46f",
	":kissing_cat:": "\U0001f63d",
	":rainbow:": "\U0001f308",
	":u5408:": "\U0001f234",
	":boot:": "\U0001f462",
	":carousel_horse:": "\U0001f3a0",
	":fried_shrimp:": "\U0001f364",
	":lock:": "\U0001f512",
	":non-potable_water:": "\U0001f6b1",
	":o:": "\U00002b55",
	":persevere:": "\U0001f623",
	":diamond_shape_with_a_dot_inside:": "\U0001f4a0",
	":fallen_leaf:": "\U0001f342",
	":massage:": "\U0001f486",
	":volcano:": "\U0001f30b",
	":gem:": "\U0001f48e",
	":shower:": "\U0001f6bf",
	":speaker:": "\U0001f508",
	":last_quarter_moon_with_face:": "\U0001f31c",
	":mag:": "\U0001f50d",
	":anguished:": "\U0001f627",
	":monkey_face:": "\U0001f435",
	":sunny:": "\U00002600",
	":tangerine:": "\U0001f34a",
	":point_right:": "\U0001f449",
	":railway_car:": "\U0001f683",
	":triumph:": "\U0001f624",
	":two:": "\U00000032\U000020e3",
	":gift_heart:": "\U0001f49d",
	":ledger:": "\U0001f4d2",
	":sagittarius:": "\U00002650",
	":snowflake:": "\U00002744",
	":abc:": "\U0001f524",
	":horse:": "\U0001f434",
	":ok_hand:": "\U0001f44c",
	":video_camera:": "\U0001f4f9",
	":sparkling_heart:": "\U0001f496",
	":taurus:": "\U00002649",
	":frog:": "\U0001f438",
	":hamster:": "\U0001f439",
	":helicopter:": "\U0001f681",
	":fries:": "\U0001f35f",
	":mushroom:": "\U0001f344",
	":penguin:": "\U0001f427",
	":truck:": "\U0001f69a",
	":bar_chart:": "\U0001f4ca",
	":evergreen_tree:": "\U0001f332",
	":bow:": "\U0001f647",
	":clock12:": "\U0001f55b",
	":four_leaf_clover:": "\U0001f340",
	":inbox_tray:": "\U0001f4e5",
	":smirk_cat:": "\U0001f63c",
	":two_men_holding_hands:": "\U0001f46c",
	":water_buffalo:": "\U0001f403",
	":alien:": "\U0001f47d",
	":video_game:": "\U0001f3ae",
	":candy:": "\U0001f36c",
	":page_facing_up:": "\U0001f4c4",
	":watermelon:": "\U0001f349",
	":white_check_mark:": "\U00002705",
	":blossom:": "\U0001f33c",
	":crocodile:": "\U0001f40a",
	":no_mouth:": "\U0001f636",
	":o2:": "\U0001f17e",
	":shirt:": "\U0001f455",
	":clock8:": "\U0001f557",
	":eyes:": "\U0001f440",
	":rabbit2:": "\U0001f407",
	":tanabata_tree:": "\U0001f38b",
	":wrench:": "\U0001f527",
	":es:": "\U0001f1ea\U0001f1f8",
	":trophy:": "\U0001f3c6",
	":two_women_holding_hands:": "\U0001f46d",
	":clock630:": "\U0001f561",
	":pineapple:": "\U0001f34d",
	":stuck_out_tongue:": "\U0001f61b",
	":angry:": "\U0001f620",
	":athletic_shoe:": "\U0001f45f",
	":cookie:": "\U0001f36a",
	":flags:": "\U0001f38f",
	":game_die:": "\U0001f3b2",
	":bird:": "\U0001f426",
	":jack_o_lantern:": "\U0001f383",
	":ox:": "\U0001f402",
	":paperclip:": "\U0001f4ce",
	":sleepy:": "\U0001f62a",
	":astonished:": "\U0001f632",
	":back:": "\U0001f519",
	":closed_book:": "\U0001f4d5",
	":hatching_chick:": "\U0001f423",
	":arrows_clockwise:": "\U0001f503",
	":car:": "\U0001f697",
	":ear:": "\U0001f442",
	":haircut:": "\U0001f487",
	":icecream:": "\U0001f366",
	":bust_in_silhouette:": "\U0001f464",
	":diamonds:": "\U00002666",
	":no_good:": "\U0001f645",
	":pizza:": "\U0001f355",
	":chicken:": "\U0001f414",
	":eyeglasses:": "\U0001f453",
	":see_no_evil:": "\U0001f648",
	":earth_africa:": "\U0001f30d",
	":fireworks:": "\U0001f386",
	":page_with_curl:": "\U0001f4c3",
	":rice_ball:": "\U0001f359",
	":white_square_button:": "\U0001f533",
	":cake:": "\U0001f370",
	":red_car:": "\U0001f697",
	":tm:": "\U00002122",
	":unamused:": "\U0001f612",
	":fish_cake:": "\U0001f365",
	":key:": "\U0001f511",
	":speedboat:": "\U0001f6a4",
	":closed_umbrella:": "\U0001f302",
	":pear:": "\U0001f350",
	":satellite:": "\U0001f4e1",
	":scream:": "\U0001f631",
	":first_quarter_moon:": "\U0001f313",
	":jp:": "\U0001f1ef\U0001f1f5",
	":repeat_one:": "\U0001f502",
	":shell:": "\U0001f41a",
	":interrobang:": "\U00002049",
	":trident:": "\U0001f531",
	":u55b6:": "\U0001f23a",
	":atm:": "\U0001f3e7",
	":door:": "\U0001f6aa",
	":kissing:": "\U0001f617",
	":six_pointed_star:": "\U0001f52f",
	":thumbsup:": "\U0001f44d",
	":u6708:": "\U0001f237",
	":do_not_litter:": "\U0001f6af",
	":whale2:": "\U0001f40b",
	":school_satchel:": "\U0001f392",
	":cactus:": "\U0001f335",
	":clipboard:": "\U0001f4cb",
	":dizzy:": "\U0001f4ab",
	":waxing_gibbous_moon:": "\U0001f314",
	":camera:": "\U0001f4f7",
	":capital_abcd:": "\U0001f520",
	":leaves:": "\U0001f343",
	":left_luggage:": "\U0001f6c5",
	":bamboo:": "\U0001f38d",
	":bowling:": "\U0001f3b3",
	":eight:": "\U00000038\U000020e3",
	":kimono:": "\U0001f458",
	":left_right_arrow:": "\U00002194",
	":stuck_out_tongue_winking_eye:": "\U0001f61c",
	":surfer:": "\U0001f3c4",
	":sweat:": "\U0001f613",
	":violin:": "\U0001f3bb",
	":postbox:": "\U0001f4ee",
	":bride_with_veil:": "\U0001f470",
	":recycle:": "\U0000267b",
	":station:": "\U0001f689",
	":vhs:": "\U0001f4fc",
	":crossed_flags:": "\U0001f38c",
	":memo:": "\U0001f4dd",
	":no_entry:": "\U000026d4",
	":white_circle:": "\U000026aa",
	":arrow_lower_left:": "\U00002199",
	":chestnut:": "\U0001f330",
	":crystal_ball:": "\U0001f52e",
	":last_quarter_moon:": "\U0001f317",
	":loud_sound:": "\U0001f50a",
	":strawberry:": "\U0001f353",
	":worried:": "\U0001f61f",
	":circus_tent:": "\U0001f3aa",
	":weary:": "\U0001f629",
	":bathtub:": "\U0001f6c1",
	":snake:": "\U0001f40d",
	":grin:": "\U0001f601",
	":symbols:": "\U0001f523",
	":airplane:": "\U00002708",
	":heart_eyes:": "\U0001f60d",
	":sailboat:": "\U000026f5",
	":stew:": "\U0001f372",
	":tshirt:": "\U0001f455",
	":rat:": "\U0001f400",
	":black_medium_square:": "\U000025fc",
	":clock930:": "\U0001f564",
	":full_moon_with_face:": "\U0001f31d",
	":japanese_goblin:": "\U0001f47a",
	":restroom:": "\U0001f6bb",
	":vertical_traffic_light:": "\U0001f6a6",
	":basketball:": "\U0001f3c0",
	":cherry_blossom:": "\U0001f338",
	":low_brightness:": "\U0001f505",
	":pill:": "\U0001f48a",
}

func Emojize(x string) string {
    result := x

    str, ok := CodeMap[string(x)]
    switch {
    case !ok:
        log.Printf("Wrong emoji syntax: %c", x)
    default:
        result = str+ReplacePadding
    }
    return result
}

// Handle state after meeting one '@'
func compileEmojiSyntax(input, output *bytes.Buffer) {
    i, _, err := input.ReadRune()
    if err != nil {
        // EOF got
        log.Print("Parse failed on emoji syntax")
        return
    }

    switch i {
    default:
        output.WriteString(string(i))
    case '{':
        emoji := bytes.NewBufferString("")
        for {
            i, _, err := input.ReadRune()
            if err != nil {
                log.Print("Parse failed on emoji syntax")
                break
            }
            if i == '}' {
                break
            }
            emoji.WriteRune(i)
        }
        output.WriteString(Emojize(emoji.String()))
    }
}

func compile(x string) string {
    if x == "" {
        return ""
    }

    input := bytes.NewBufferString(x)
    output := bytes.NewBufferString("")

    for {
        i, _, err := input.ReadRune()
        if err != nil {
            break
        }
        switch i {
        default:
            output.WriteRune(i)
        case EscapeChar:
            compileEmojiSyntax(input, output)
        }
    }
    return output.String()
}

func compileValues(a *[]interface{}) {
    for i, x := range *a {
        if str, ok := x.(string); ok {
            (*a)[i] = compile(str)
        }
    }
}

func Print(a ...interface{}) (int, error) {
    compileValues(&a)
    return fmt.Print(a...)
}

func Println(a ...interface{}) (int, error) {
    compileValues(&a)
    return fmt.Println(a...)
}

func Printf(format string, a ...interface{}) (int, error) {
    format = compile(format)
    return fmt.Printf(format, a...)
}

func Fprint(w io.Writer, a ...interface{}) (int, error) {
    compileValues(&a)
    return fmt.Fprint(w, a...)
}

func Fprintln(w io.Writer, a ...interface{}) (int, error) {
    compileValues(&a)
    return fmt.Fprintln(w, a...)
}

func Fprintf(w io.Writer, format string, a ...interface{}) (int, error) {
    format = compile(format)
    return fmt.Fprintf(w, format, a...)
}

func Sprint(a ...interface{}) string {
    compileValues(&a)
    return fmt.Sprint(a...)
}

func Sprintf(format string, a ...interface{}) string {
    format = compile(format)
    return fmt.Sprintf(format, a...)
}

func Errorf(format string, a ...interface{}) error {
    return errors.New(Sprintf(format, a...))
}

